#procedure DoSynch(i)
****************************************************************************
**
**  Filename:
**            DoSynch.prc
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose: 
**            Procedure to synchronize the arguments of 
**            \keyword{num}, \keyword{den}, \keyword{fac}, \keyword{invfac} 
**            and \keyword{S}. The argument specifies 
**            the index $i$ of the sum to be adjusted.
**            Start with synchronizing products of \keyword{S} functions, 
**            then do the combinations 
**            \begin{enumerate}
**            \item $j_i$ and \keyword{fac}, \keyword{invfac}
**            \item \keyword{den} and \keyword{S}
**            \item \keyword{den} and \keyword{fac}, \keyword{invfac}
**            \item \keyword{S} and \keyword{fac}, \keyword{invfac}.
**            \end{enumerate}
**            Finally summation boundaries are synchronized.
**
**  Comments:
**            This procedure is a modified version of a similiar procedure  
**            in the Summer package written by J.A.M. Vermaseren.
**
**  Revision: 
**            $Id: DoSynch.prc,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: DoSynch.prc,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*
id  S(R,?a) = 1;
id  S`i'(R,?a) = 1;
id pow(0,?x) = 0;
id pow(1,?x) = 1;
id pow(x?,0) = 1;
id pow`i'(0,?x) = 0;
id pow`i'(1,?x) = 1;
id pow`i'(x?,0) = 1;
#call SynchSS(`i')
id  S(R,?a) = 1;
id  S`i'(R,?a) = 1;

SplitArg,(j`i'),den;
id den(j`i') = den`i'(0,j`i');
id den(-j`i') = -den`i'(0,j`i');
id den(x?,j`i') = den`i'(x,j`i');
id den(x?,-j`i') = den`i'(x,-j`i');

#do ii = `i',1,-1
  #call NormDen(`ii')
#enddo

#do ii = `i',1,-1
  #call SplitDen(`ii')
#enddo
*
*
*   First see whether we need to invert the direction of summation.
*   (This is needed if j`i' in den comes with a negative sign and
*   the difference between the offset in den and the upper limit is
*   just an integer.
*
id  sum`i'(j`i',x1?,x2?)*den`i'(x3?,-j`i') =
        X(x3-x2)*sum`i'(j`i',x1,x2)*den`i'(x3,-j`i');
if ( match(X(x1?int_)) );
    #call Flipj(`i')
endif;
id  X(x1?) = 1;
.sort:SynchFlipDir`i'-a;
*
*   Then see whether we need to move the summation a bit.
*
if ( match(den`i'(x1?pos_,j`i')) );
    id,once,den`i'(x1?pos_,j`i') = den`i'(x1,j`i')*X(x1);
    #call Movej(`i',X)
    id  X(x1?) = 1;
endif;
id  S(R,?a) = 1;
id  S`i'(R,?a) = 1;
.sort:SynchShiftDen`i';
*
*   Now comes the great recursion
*

#do k1 = 1,1
#do k2 = 1,1
*
*   1: combinations of j`i' and fac,invfac
*
#call SynchNumFac(`i')
*
*   2: combinations of den and S.
*
#call SynchDenS(`i')
*
*   3: combinations of den and fac,invfac.
*
#call SynchDenFac(`i')

id  S(R,X,?a) = 1;
id  S`i'(R,X,?a) = 1;
id pow(0,?x) = 0;
id pow(1,?x) = 1;
id pow(x?,0) = 1;
id pow`i'(0,?x) = 0;
id pow`i'(1,?x) = 1;
id pow`i'(x?,0) = 1;
*
*   If there was action we have to restart with den and S.
*
if ( $syndf > 0 ) redefine k2 "0";
*
.sort:SynchDenFac`i';
#enddo
*
*       Combination of j and fac or invfac
*       If there is a j there is no den!
*
if ( count(j`i',1) > 0 );
        repeat id j`i'*invfac`i'(n?pos_,j`i') =
                                invfac`i'(n-1,j`i')-n*invfac`i'(n,j`i');
endif;
*
*   4: combinations of fac,invfac and S
*
#call SynchFacS(`i')
id  S(R,?a) = 1;
id  S`i'(R,?a) = 1;
id pow(0,?x) = 0;
id pow(1,?x) = 1;
id pow(x?,0) = 1;
id pow`i'(0,?x) = 0;
id pow`i'(1,?x) = 1;
id pow`i'(x?,0) = 1;
*
*   If there was action, we have to start from the beginning again.
*

if ( $synfs > 0 ) redefine k1 "0";

#if `$synfs' > 0
#do mii = `i',1,-1
#call SplitDen(`mii')
#enddo

.sort:SynchSplitDen;
#do mii = `i',1,-1
#call DoTheta(`mii')
#enddo
#call DoTheta(0)
.sort:SynchDoTheta`i';
#call SynchNumFac(`i')
.sort:SynchNumFac`i';
#endif
id  S(R,X,?a) = 1;
id  S`i'(R,X,?a) = 1;
id pow(0,?x) = 0;
id pow(1,?x) = 1;
id pow(x?,0) = 1;
id pow`i'(0,?x) = 0;
id pow`i'(1,?x) = 1;
id pow`i'(x?,0) = 1;

id  sum`i'(j`i',x1?,x2?)*den`i'(x3?,-j`i') =
        XX(x3-x2)*sum`i'(j`i',x1,x2)*den`i'(x3,-j`i');
if ( match(XX(x1?int_)) );
    #call Flipj(`i')
else if ( ( count(den`i',1) == 0 ) && ( count(S`i',1) == 1 ) );
    id  sum`i'(j`i',x1?,x2?)*S`i'(?a,x3?,-j`i') =
            XX(x3-x2)*sum`i'(j`i',x1,x2)*S`i'(?a,x3,-j`i');
    if ( match(XX(x1?int_)) );
        #call Flipj(`i')
    endif;
endif;
id  XX(x1?) = 1;
#call SynchDenS(`i')
id  sum`i'(j`i',n1?,n2?) = XX(n1,n2,n2-n1);
if ( match(XX(n1?,n2?,n3?int_)) );
    id XX(n1?,n2?,n3?neg_) = 0;
    repeat;
        id  XX(n1?,n2?,n3?pos_) = replace_(j`i',n2)+XX(n1,n2-1,n3-1);
    endrepeat;
    id  XX(n1?,n2?,0) = replace_(j`i',n2);
    id  S`i'(R(?a),X(?x),n1?,n2?) = S(R(?a),X(?x),n1+n2);
    id  bino`i'(n1?,n2?,n3?) = fac_(n1)*invfac_(n2+n3)*invfac_(n1-n2-n3);
    id  fac`i'(n1?,n2?) = fac_(n1+n2);
    id  invfac`i'(n1?,n2?) = invfac_(n1+n2);
    id  fac_(x?) = fac(x);
    id  invfac_(x?) = invfac(x);
    id  pow`i'(x1?,n1?) = pow(x1,n1);
    id  den`i'(n1?,n2?) = den(n1+n2);
else;
    id  XX(n1?,n2?,n3?) = sum`i'(j`i',n1,n2);
endif;
.sort:SynchFlipDir`i'-b;
*
* end of the great recursion
*
#enddo

*
*   5: synchronize with the summation boundaries
*      also introduces the binomial coefficients
*
#call AdjustSum(`i')

#endprocedure
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************


