#procedure SplitDen(i)
****************************************************************************
**
**  Filename:
**            
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose: 
**            Partial fractioning of products of denominators involving
**            the summation variable $j_i$, where $i$ is the argument passed
**            to \keyword{SplitDen}. 
**            This routine is optimized for higher powers of the denominators 
**            and uses the sum formula rather than repeated splitting of pairs. 
**
**  Comments:
**
**
**  Revision: 
**            $Id: SplitDen.prc,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: SplitDen.prc,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*
*   We do j`i'
*   First put the den functions to one argument notation.
*
id f?{den,den0,...,den`i'}(x1?,x2?) = den(x1+x2);
id f?{den0,...,den`i'}(x1?) = den(x1);
*
*   Single out the j`i'
*
SplitArg,(j`i'),den;
id  den(x?,j`i') = den`i'(1,x,j`i');
id  den(x?,-j`i') = den`i'(1,x,-j`i');
id  den(j`i') = den`i'(1,0,j`i');
id  den(-j`i') = -den`i'(1,0,j`i');
*
*   Now the first parameter is the power.
*   We recognize only cases with j and -j.
*   Multiples we don't use. (not needed for two loop structure functions)
*   Now do some simplifications with the numerators.
*
repeat id j`i'*den`i'(1,0,j`i') = 1;
repeat id j`i'*den`i'(1,x?,j`i') = 1-x*den`i'(1,x,j`i');
*
*   Collect the powers
*
repeat id,den`i'(n1?,x1?,x2?)*den`i'(n2?,x1?,x2?) = den`i'(n1+n2,x1,x2);
*
*   The cases den(x+j) and den(x-j) are considered separately, because
*   there is ordering according to the 'biggest' objects
*   Hence we need three formula's
*   The function deltap_(x) = 1-delta_(x)
*   It will be replaced later by two theta functions.
*
repeat;
    id  den`i'(n1?,x1?,j`i')*den`i'(n2?,x2?,j`i') =
        +sum_(jjj,1,n1,binom_(n1+n2-1-jjj,n2-1)*den`i'(jjj,x1,j`i')
                *den(x1-x2)^(n1+n2-jjj))*sign_(n2)*deltap_(x1-x2)
        +sum_(jjj,1,n2,binom_(n1+n2-1-jjj,n1-1)*den`i'(jjj,x2,j`i')
                *den(x2-x1)^(n1+n2-jjj))*sign_(n1)*deltap_(x2-x1)
        +delta_(x1-x2)*den`i'(n1+n2,x1,j`i');
    id  den(x?pos_) = 1/x;
    id  den(x?neg_) = 1/x;
endrepeat;
repeat;
    id  den`i'(n1?,x1?,-j`i')*den`i'(n2?,x2?,-j`i') =
        +sum_(jjj,1,n1,binom_(n1+n2-1-jjj,n2-1)*den`i'(jjj,x1,-j`i')
                *den(x1-x2)^(n1+n2-jjj))*sign_(n2)*deltap_(x1-x2)
        +sum_(jjj,1,n2,binom_(n1+n2-1-jjj,n1-1)*den`i'(jjj,x2,-j`i')
                *den(x2-x1)^(n1+n2-jjj))*sign_(n1)*deltap_(x2-x1)
        +delta_(x1-x2)*den`i'(n1+n2,x1,-j`i');
    id  den(x?pos_) = 1/x;
    id  den(x?neg_) = 1/x;
endrepeat;
repeat;
    id  den`i'(n1?,x1?,j`i')*den`i'(n2?,x2?,-j`i') =
        +sum_(jjj,1,n1,binom_(n1+n2-1-jjj,n2-1)*den`i'(jjj,x1,j`i')
                *den(x1+x2)^(n1+n2-jjj))*deltap_(x1+x2)
        +sum_(jjj,1,n2,binom_(n1+n2-1-jjj,n1-1)*den`i'(jjj,x2,-j`i')
                *den(x2+x1)^(n1+n2-jjj))*deltap_(x1+x2)
        +delta_(x1+x2)*den`i'(n1+n2,x1,j`i')*sign_(n2);
    id  den(x?pos_) = 1/x;
    id  den(x?neg_) = 1/x;
endrepeat;
*
*   Cleanup. Go to the 'own' functions.
*
id  den`i'(n1?,x1?,x2?) = den`i'(x1,x2)^n1;
id  delta_(x?) = delta(x);
id  deltap_(x?) = deltap(x);
*
#endprocedure
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************
