#procedure SynchDenFac(i)
****************************************************************************
**
**  Filename:
**            SynchDenFac.prc
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose: 
**            Synchronizes combinations of \keyword{den} and \keyword{fac}, 
**            \keyword{invfac} functions, i.e. factorials and denominators. 
**            The argument specifies the associated summation index $i$.
**
**  Comments:
**            This procedure is a modified version of a similiar procedure 
**            in the Summer package written by J.A.M. Vermaseren.
**
**            The variable $syndf will be 1 if anything relevant happened.
**
**  Revision: 
**            $Id: SynchDenFac.prc,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: SynchDenFac.prc,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*
#$syndf = 0;
repeat;
    id,once,den`i'(x1?,x?)*fac`i'(x2?,x?) = f1(1,x1,x2,x,x2-x1);
    if ( match(f1(1,x1?,x2?,x3?,n?int_)) );
        repeat id f1(n1?,x1?,x2?,x?,x3?)*den`i'(x1?,x?) = f1(n1+1,x1,x2,x,x3);
        id  f1(?a) = f2(?a);
    else;
        id  f1(1,x1?,x2?,x?,x3?) = den`i'(x1,x)*f3(x2,x);
    endif;
endrepeat;
id  f3(?a) = fac`i'(?a);
repeat;
    id,once,den`i'(x1?,x?)*invfac`i'(x2?,x?) = f1(1,x1,x2,x,x2-x1);
    if ( match(f1(1,x1?,x2?,x3?,n?int_)) );
        repeat id f1(n1?,x1?,x2?,x?,x3?)*den`i'(x1?,x?) = f1(n1+1,x1,x2,x,x3);
        id  f1(?a) = f4(?a);
    else;
        id  f1(1,x1?,x2?,x?,x3?) = den`i'(x1,x)*f3(x2,x);
    endif;
endrepeat;
id  f3(?a) = invfac`i'(?a);
if ( match(f2(1,x1?,x2?,x?,0)) );
    id  f2(1,x1?,x2?,x?,0) = fac`i'(x2-1,x);
    $syndf = 1;
endif;
id  f2(n?,x1?,x2?,x?,0) = den`i'(x1,x)^n*fac`i'(x2,x);
id  f4(n?,x1?,x2?,x?,0) = den`i'(x1,x)^n*invfac`i'(x2,x);
*
*   f2 contains a combination of den and fac
*   f4 contains a combination of den and invfac
*
if ( count(f2,1,f4,1) );
    $syndf = 1;
#if `i' == 0
    Print "sych0: %t";
#endif
endif;

repeat;
    id  f2(n?pos_,x1?,x2?,x?,n1?pos_) =
            f2(n-1,x1,x2-1,x,n1-1) + n1*f2(n,x1,x2-1,x,n1-1);
    id  f2(1,x1?,x2?,x?,0) = fac`i'(x2-1,x);
    id  f2(n?,x1?,x2?,x?,0) = den`i'(x1,x)^n*fac`i'(x2,x);
    id  f2(0,x1?,x2?,x?,n1?) = fac`i'(x2,x);
endrepeat;
repeat;
    id  f2(n?,x1?,x2?,x?,-1) = fac`i'(x1,x)*den`i'(x1,x)^(n+1);
    id  f2(n?pos_,x1?,x2?,x?,n1?neg_) = fac`i'(x2,x)/(-n1-1)^n
            -sum_(jjj,1,n,f2(jjj,x1,x2+1,x,n1+1)/(-n1-1)^(n+1-jjj));
endrepeat;
repeat;
    id  f4(n?,x1?,x2?,x?,-1) = invfac`i'(x1,x)*den`i'(x1,x)^(n-1);
    id  f4(n?pos_,x1?,x2?,x?,n1?neg_) =
            f4(n-1,x1,x2+1,x,n1+1) + acc1(n1+1)*f4(n,x1,x2+1,x,n1+1);
	id	acc1(n?) = n;
    id  f4(0,x1?,x2?,x?,n1?) = invfac`i'(x2,x);
endrepeat;
repeat;
    id  f4(n?,x1?,x2?,x?,0) = den`i'(x1,x)^n*invfac`i'(x2,x);
    id  f4(n?pos_,x1?,x2?,x?,n1?pos_) = invfac`i'(x2,x)/n1^n*sign_(n)
            +sum_(jjj,1,n,f4(jjj,x1,x2-1,x,n1-1)/n1^(n+1-jjj)*sign_(n-jjj));
endrepeat;

#endprocedure
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************
